<?php

defined('MOODLE_INTERNAL') || die;

interface i_attribute_manager {

    public static function set_attribute($course, $attr_type, $name, $value);

    public static function get_attribute($course, $name);
}


class attribute_manager implements i_attribute_manager {

    private static $attribute_types = array(
    	PARAM_INT => 'integer',
    	PARAM_ALPHANUMEXT => 'string',
    	PARAM_BOOL => 'boolean',
      PARAM_TEXT => 'string',
    );

    private static function get_type_name($attr_type)
    {
	foreach (self::$attribute_types as $type => $type_name) {
	    if ($type == $attr_type)
		return ($type_name);
	}

	return null;
    }

    public static function set_attribute($course, $attr_type, $name, $value)
    {
      	global $DB;

        $type = self::get_type_name($attr_type);
        if ($type == NULL)
            throw new moodle_exception('unknownattributetype', 'local_stag', '', $attr_type);

        $transaction = $DB->start_delegated_transaction();

        settype($value, "string");
        $rec = $DB->get_record("course_attributes", array("course_id" => $course, "name" => $name));
        if ($rec) {
            if ($rec->type != $type)
                throw new moodle_exception('conflictingttributetype', 'local_stag', '', $attr_type);
            $rec->value = $value;
  	        $DB->update_record("course_attributes", $rec);
          } else {
              $rec = new stdClass();
              $rec->course_id = $course;
              $rec->type = $type;
              $rec->name = $name;
              $rec->value = $value;
  	          $DB->insert_record("course_attributes", $rec);
          }
       $transaction->allow_commit();
       return 0;
    }

    public static function get_attribute($course, $name)
    {
	global $DB;

	$rec = $DB->get_record("course_attributes", array("course_id" => $course, "name" => $name));
	if (! $rec)
	    return null;

        settype($rec->value, $rec->type);
        return  $rec->value;
    }

		/**
		 *
		 * @global moodle_database $DB
		 * @param integer $course_id
		 * @param string $attribute_name
		 */
		public static function delete_attribute($course_id, $attribute_name){
			global $DB;
			return $DB->delete_records('course_attributes',array('course_id'=>$course_id,'name'=>$attribute_name));
		}
}
