<?php

defined('MOODLE_INTERNAL') || die();

global $CFG;

require_once($CFG->dirroot . '/local/stag/externallib.php');
require_once($CFG->dirroot . '/webservice/tests/helpers.php');

/**
  * @backupStaticAttributes enabled
  */
class stag_course_test extends externallib_advanced_testcase {

    private $stag_course;

    private static $new_course = array(
            "categoryid" => 1,
            "shortname"  => "testovac5",
            "full_name"  => "Velky testan",
            "stag_id"    => array(array(
                "department" => "DPT",
                "subject"    => "Macaty_kurz",
                "year"       => 2010)),
            "annotation" => "Lorem Ipsum",
            "start_date" => 2010);

    protected function setUp() {
        $this->stag_course = new local_stag_course_external();

        $this->resetAfterTest(true);

        /* XXX not quite nice, but it just works. Any way how to work around the static property is appreciated */
        //attribute_manager::$field_category = NULL;
    }

    private function gen_template() {
        $cat = $this->getDataGenerator()->create_category(array('name' => 'Course Template Category'));
        $template = $this->getDataGenerator()->create_course(array('category' => $cat->id, 'shortname'=>'Template'));

        return $template;
    }

    private function gen_previous() {
        $previous = $this->getDataGenerator()->create_course(array('summary' => 'Prdelacka'));

        $course_id = $previous->id;
        $stag_id = self::$new_course['stag_id'][0];
        attribute_manager::set_attribute($course_id, PARAM_ALPHANUMEXT, 'department', $stag_id['department']);
        attribute_manager::set_attribute($course_id, PARAM_ALPHANUMEXT, 'subject', $stag_id['subject']);
        attribute_manager::set_attribute($course_id, PARAM_INT, 'year', $stag_id['year']-1);

        return $previous;
    }

    private function catch_exception($new_course, $errcode) {
        try {
            $this->stag_course->stag_create_course($new_course);
        } catch (moodle_exception $e) {
            if ($e->errorcode == $errcode)
	        return 0;
        }
	$this->fail('The expected exception has not been thrown.');
    }

    private function set_perms() {
        global $USER;

        $this->setAdminUser();
        /* as per the tests for course we need to fill in some email address */
        $USER->email = 'emailtopass@contextvalidation.me';

        /*
        $contextid = context_system::instance()->id;
        $this->assignUserCapability('moodle/course:create', $contextid);
        $this->assignUserCapability('enrol/guest:config', $contextid);
        */
    }

    private function check_course_contents($new_course_id, $template_id, $previous_course) {
        global $DB;

        $new_course = $DB->get_course('course', array('id' => $new_course_id), '*');
        $template = $DB->get_course('course', array('id' => $template_id), '*');
        
    }

    /*
     * Actual tests follow
     */

    public function test_missing_template() {
        $this->set_perms();

        $this->catch_exception(self::$new_course, 'missingtemplate');
    }

    public function test_create_perms() {
   //     $this->resetAfterTest(true);

        $this->setExpectedException('required_capability_exception');
        $this->stag_course->stag_create_course(self::$new_course);
    }

    /* XXX make it return stag_id and roakidno */
    public function get_stag_params() {
	return array(
	    array(null, null, null),
//	    array(undef, 'subj', 123));
            );
    }

    /**
      * @dataProvider get_stag_params
      */
    public function test_stag_id($department, $subject, $year) {
        $this->set_perms();

	$new_course = self::$new_course;
	$new_course['department'] = $department;
	$new_course['subject'] = $subject;
	$new_course['year'] = $year;

	$this->catch_exception('einval', $new_course);
    }

    public function test_params() {
	$this->set_perms();

        $new_course = self::$new_course;

        /* try with both stag_id and roakidno set */
        $new_course['stag_roakidno'] = "Ramboidno";
        $this->catch_exception('clashingparams', $new_course);

        /* try with no stag_id set 
        unset($new_course['stag_id']);
        $id = $this->stag_course->stag_create_course($new_course);
        $this->assertGreaterThan(0, $id);
        */

        /*
        $new_course['shortname'] = $new_course['shortname'] . "X";
        $new_course['stag_id'] = array();
        $this->catch_exception('einval', $new_course);
         * 
         */
    }

    public function test_create_from_template() {
	$this->set_perms();

        $template = $this->gen_template();

        $this->stag_course->stag_create_course(self::$new_course);
    }

    public function test_create_from_previous() {
        global $DB;

	$this->set_perms();

        $this->gen_template();
        $previous = $this->gen_previous();

        $course_id = $this->stag_course->stag_create_course(self::$new_course);

        $dep = attribute_manager::get_attribute($course_id, 'department');
        $this->assertSame($dep, "DPT");

        $course = $DB->get_record('course', array('id' => $course_id));
        $this->assertSame($course->summary, $previous->summary);
    }

    public function test_get_course() {
	$this->set_perms();

        $course = $this->getDataGenerator()->create_course();
        attribute_manager::set_attribute($course->id, PARAM_ALPHANUMEXT, 'department', "Katedra");
        attribute_manager::set_attribute($course->id, PARAM_INT, 'year', 1111);

        $courseids = array($course->id);
        $result = $this->stag_course->stag_get_courses(array('ids' => $courseids));

        $this->assertSame($result[0]['stag_department'], "Katedra");
    }
}
